--
-- Reference design - Simple LEDs on Spartan-3E Starter Kit using PicoBlaze (KCPSM3)
--
-- George Economakos - NTUA/uLab - June 2008
--
-- This design demonstrates how to interface to the push buttons and simple LEDs.
--    At the start, only one LED is on. 
--    Pressing the east and west buttons will cause the LED which is on to appear to 
--    also move in the corresponding direction.
--    Pressing the north button will invert all LEDs so that only one is off.
--
-- The design also uses the 50MHz oscillator provided on the board.
--
------------------------------------------------------------------------------------
--
-- Library declarations
--
-- Standard IEEE libraries
--
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
--
------------------------------------------------------------------------------------
--
--
entity left_right_leds_embed is
    Port (             
	
--TODO1--: Declare Input (buttons,clk) and Output (leds) ports 
	
	);
    end left_right_leds_embed;
--
------------------------------------------------------------------------------------
--
-- Start of test architecture
--
architecture RTL of left_right_leds_embed is
--
------------------------------------------------------------------------------------
--
-- declaration of KCPSM3
--
  component kcpsm3 
    Port (   
	--TODO2--: Declare Input and output ports of component kcpsm3	
	 
	 );
    end component;
--
-- declaration of program ROM
--
  component lrrom
 ---TODO3: Insert correct names of input ports and output port of ROM block
    Port (      ??? : in std_logic_vector(9 downto 0);
                ??? : out std_logic_vector(17 downto 0);
             --proc_reset : out std_logic;                       --JTAG Loader version
                    clk : in std_logic);
    end component;
--
------------------------------------------------------------------------------------
--
-- Signals used to connect KCPSM3 to program ROM and I/O logic
--
signal address          : std_logic_vector(9 downto 0);
signal instruction      : std_logic_vector(17 downto 0);
signal port_id          : std_logic_vector(7 downto 0);
signal out_port         : std_logic_vector(7 downto 0);
signal in_port          : std_logic_vector(7 downto 0);
signal write_strobe     : std_logic;
signal read_strobe      : std_logic;
signal interrupt        : std_logic :='0';
signal interrupt_ack    : std_logic;
signal kcpsm3_reset     : std_logic;
--
--
------------------------------------------------------------------------------------------------------------------------------------------------------------------------
--
-- Start of circuit description
--
begin
  --
  --
  ----------------------------------------------------------------------------------------------------------------------------------
  -- KCPSM3 and the program memory 
  ----------------------------------------------------------------------------------------------------------------------------------
  --

  processor: kcpsm3
    port map(      address => address,
               instruction => instruction,
                   port_id => port_id,
              write_strobe => write_strobe,
                  out_port => out_port,
               read_strobe => read_strobe,
                   in_port => in_port,
                 interrupt => interrupt,
             interrupt_ack => interrupt_ack,
                     reset => kcpsm3_reset,
                       clk => clk);
 
  program_rom: lrrom
    port map(      address => address,
               instruction => instruction,
                --proc_reset => kcpsm3_reset,                       --JTAG Loader version 
                       clk => clk);

  --
  ----------------------------------------------------------------------------------------------------------------------------------
  -- KCPSM3 input ports 
  ----------------------------------------------------------------------------------------------------------------------------------
  --
  --
  -- The inputs connect via a pipelined multiplexer
  --

  input_ports: process(clk)
  begin
    
	 if clk'event and clk='1' then

      case port_id is

        -- read buttons at address 00 hex
--TODO4: Insert in the following line between " the correct address of input port 
        when " " =>    in_port <=  "00000" & btn_west & btn_north & btn_east;

        -- Don't care used for all other addresses to ensure minimum logic implementation
        when others =>    in_port <= "XXXXXXXX";  

      end case;

    end if;

  end process input_ports;

  --
  ----------------------------------------------------------------------------------------------------------------------------------
  -- KCPSM3 output ports 
  ----------------------------------------------------------------------------------------------------------------------------------
  --

  -- adding the output registers to the processor
   
  output_ports: process(clk)
  begin

    if clk'event and clk='1' then
      if write_strobe='1' then

        -- Write to LEDs at address 80 hex.
---TODO5: put correct address of output port_id in the following line
        if port_id=" " then
          led <= out_port;
        end if;

      end if;

    end if; 

  end process output_ports;
 
  --
  --
  ----------------------------------------------------------------------------------------------------------------------------------
  --
  --
end RTL;

------------------------------------------------------------------------------------------------------------------------------------
--
-- END OF FILE left_right_leds_embed.vhd
--
------------------------------------------------------------------------------------------------------------------------------------

